/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.mvn;

import ch.hortis.sonar.mvn.reports.ReportHandler;
import org.apache.maven.plugin.MojoExecutionException;
import org.picocontainer.MutablePicoContainer;
import org.sonar.plugins.api.*;
import org.sonar.plugins.maven.MavenUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * @goal prepare
 */
public class PrepareMojo extends CoreMojo {


  protected List<Report> getTargetReports() {
    if (isLight()) {
      return Report.getLightReports();
    }
    return Report.getReports();
  }

  public void doExecute() throws MojoExecutionException {
    checkPluginsPresenceForProjectLanguage();

    MutablePicoContainer childContainer = null;
    try {
      childContainer = getContainer().makeChildContainer();
      for (Plugin plugin : getContainer().getComponents(Plugin.class)) {
        if (plugin.getMavenPluginHandlerClass() != null) {
          childContainer.addComponent(plugin.getMavenPluginHandlerClass());
        }
      }
      childContainer.start();

      for (MavenPluginHandler handler : childContainer.getComponents(MavenPluginHandler.class)) {
        if (handler.shouldExecuteOn(mavenProject)) {
          prepareMavenPlugin(handler);
        }
      }

    } catch (Exception e) {
      throw new MojoExecutionException("Error during sonar plugin access", e);

    } finally {
      if (childContainer != null) {
        //childContainer.stop();
        getContainer().removeChildContainer(childContainer);
      }
    }

    if (MavenUtils.getLanguage(mavenProject).equals(Languages.JAVA)) {
      preparePomForReports();
      generateGoals();
    }
  }

  private void prepareMavenPlugin(MavenPluginHandler handler) throws MojoExecutionException {
    handler.configure(mavenProject);
    for (String command : getCommands(handler)) {
      String goals = mavenProject.getProperties().getProperty("sonar.bootstrap.additional.goals");
      if (goals == null) {
        goals = "";
      }
      goals += command + "," + handler.shouldStopOnFailure() + ";";
      mavenProject.getProperties().setProperty("sonar.bootstrap.additional.goals", goals);
    }
  }


  private List<String> getCommands(MavenPluginHandler handler) {
    ArrayList<String> goals = new ArrayList<String>();
    String prefix = handler.getGroupId() + ":" + handler.getArtifactId() + ":" + handler.getVersion() + ":";
    for (String goal : handler.getGoals()) {
      goals.add(prefix + goal);
    }
    return goals;
  }


  private void generateGoals() throws MojoExecutionException {
    boolean hasJavaSources = !MavenUtils.getJavaSourceFiles(mavenProject).isEmpty();
    boolean hasJavaTests = !MavenUtils.getJavaTestFiles(mavenProject).isEmpty();

    for (Report report : getTargetReports()) {
      ReportHandler reportHandler = report.getReportHandler();
      if (reportHandler != null) {
        if ((hasJavaSources || reportHandler.executeEvenIfNoJavaSources())
          && (hasJavaTests || reportHandler.executeEvenIfNoJavaTests()) &&
          reportHandler.execute(mavenProject, getLog())) {
          setReportGoals(reportHandler);
        }
      }
    }
  }

  private void setReportGoals(ReportHandler reportHandler) throws MojoExecutionException {
    for (String command : reportHandler.getCommands()) {
      String goals = mavenProject.getProperties().getProperty("sonar.bootstrap.additional.goals");
      if (goals == null) goals = "";
      goals += command + "," + reportHandler.mustFailOnError() + ";";
      mavenProject.getProperties().setProperty("sonar.bootstrap.additional.goals", goals);
    }
  }

  private void preparePomForReports() throws MojoExecutionException {
    File targetDir = new File(mavenProject.getBuild().getDirectory() + "/sonar");
    if (!targetDir.exists() && !targetDir.mkdirs()) {
      throw new MojoExecutionException("Unable to created directory " + targetDir.getPath());
    }

    for (Report report : getTargetReports()) {
      ReportHandler reportHandler = report.getReportHandler();
      if (reportHandler != null) {
        reportHandler.preparePom(mavenProject);
      }
    }
    if (mavenProject.getReporting() != null) {
      mavenProject.getReporting().setPlugins(null);
    }
  }

  private void checkPluginsPresenceForProjectLanguage() {
    Language language = MavenUtils.getLanguage(mavenProject);
    PluginsIntrospector pluginsIntrospector = getContainer().getComponent(PluginsIntrospector.class);
    if (pluginsIntrospector.getPluginsByLanguage(language.getKey()).size() == 0) {
      throw new RuntimeException("No plugins available for the language : " + language.getName());
    }
  }
}